/*
Dozen Days of Dice episode 2: Thirty Six
Copyright (c) 2007-2008 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.dice.thirtysix
{
	// Adobe Imports
//	import flash.display.SimpleButton;
	import flash.display.MovieClip;
	import flash.events.Event;
//	import flash.events.MouseEvent;
	
	// Blazing Game imports
//	import com.blazinggames.dozendays.dice.*;
	
	/**
	 * Manages the game of thirty six
	 *
	 * @author Billy D. Spelchan
	 * @version 1.0
	 * Copyright (c) 2007-2008 Blazing Games Inc. All Rights Reserved.
	*/
	public class ThirtySixManager
	extends MovieClip
	{
// *********************
// ***** CONSTANTS *****
// *********************

		public const NUM_PLAYERS = 6;
		public const STATE_INITIALIZING:int = 0;
		public const STATE_STARTING:int = 1;
		public const STATE_PLAYING:int = 2;
		public const STATE_FINISHING:int = 3;

// *********************
// ***** VARIABLES *****
// *********************

		protected var _playerArray:Array;
		protected var _gameState:int;
		protected var _currentPlayer:int;
		
// ***********************
// ***** CONSTRUCTOR *****
// ***********************

		public function ThirtySixManager()
		{
			_playerArray = new Array(
				redPlayer_movie, yellowPlayer_movie, greenPlayer_movie,
				cyanPlayer_movie, bluePlayer_movie, purplePlayer_movie
			);
			_gameState = STATE_INITIALIZING;
			for (var cntr:int = 0; cntr < 6; ++cntr)
				_playerArray[cntr].addEventListener(
						redPlayer_movie.PLAYER_DONE_EVENT, 
						handlePlayerStateChange);
			startRound(true);
		}
		
// **************************************
// ***** Functions - Event handlers *****
// **************************************

		public function handlePlayerStateChange(evt:Event)
		{
			if (_gameState != STATE_PLAYING)
				return;
			var doneCount:int = 0;
			var foundPlayer:Boolean = false;
			do {
				++_currentPlayer;
				if (_currentPlayer >= NUM_PLAYERS) // wrap players
					_currentPlayer = 0; 
				foundPlayer = _playerArray[_currentPlayer].getPlayerState()==0;
				++doneCount;
			} while ((foundPlayer == false) && (doneCount < NUM_PLAYERS));
			if (foundPlayer == false)
				endRound();
			else
				_playerArray[_currentPlayer].setPlayerState(
						redPlayer_movie.STATE_PLAYING);
		}


// ***************************************
// ***** Functions - General control *****
// ***************************************

		public function startRound(reset:Boolean = false)
		{
			var cntr:int;
			
			for (cntr = 0; cntr < NUM_PLAYERS; ++cntr)
				_playerArray[cntr].reset(reset);

			_gameState = STATE_PLAYING;
			_currentPlayer = 0;
			_playerArray[_currentPlayer].setPlayerState(
					redPlayer_movie.STATE_PLAYING);
		}
		
		protected function endRound()
		{
			var cntr:int, temp:int;
			var highScore:int = 0;
			var tie:Boolean = false;

			// first pass to find high score and if tie
			for (cntr = 0; cntr < NUM_PLAYERS; ++cntr) {
				temp = _playerArray[cntr].getScore();
				if ((temp <= 36) && (temp > highScore)) {
					tie = false;
					highScore = temp;
				} else if (temp == highScore) {
					tie = true;
				}
			}

			// second pass to actually declair winner(s)
			if (tie)
				temp = redPlayer_movie.STATE_TIE;
			else
				temp = redPlayer_movie.STATE_WIN;
			for (cntr = 0; cntr < NUM_PLAYERS; ++cntr) {
				if (_playerArray[cntr].getScore() == highScore) {
					_playerArray[cntr].setPlayerState(temp);
				}
			}
		}
	} // end interface
} // end package

