﻿/*
Dozen Days of Dice episode 2: Thirty Six
Copyright (c) 2007-2008 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.dice.thirtysix
{
	// Adobe Imports
	import flash.display.SimpleButton;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	
	// Blazing Game imports
	import com.blazinggames.dozendays.dice.*;
	
	/**
	 * A single instance of the game. Six of these instances make up the actual
	 * game, though for testing only one is needed.
	 *
	 * @author Billy D. Spelchan
	 * @version 1.0
	 * Copyright (c) 2007-2008 Blazing Games Inc. All Rights Reserved.
	*/
	public class ThirtySixPlayer
	extends MovieClip
	{
// *********************
// ***** CONSTANTS *****
// *********************

		public const PLAYER_DONE_EVENT:String = "36PlayerDone";
		public const AI_THRESHOLD:int = 33;
		
		// layout constants
		protected const DIE_LIMITS:Array = new Array(5, 20, 40);
		protected const DIE_NUMROWS:Array = new Array(1, 2, 3);
		protected const DIE_DICEPERROW:Array = new Array(5,10, 14);
		protected const DIE_SIZE:Array = new Array(64, 32, 24);
		protected const OFFSET_X = 150;
		protected const OFFSET_Y = 1;
		protected const HGAP = 1;
		protected const VGAP = 1;

		// frame label constants
		protected const LABELS:Array = new Array(
				"waiting", "input", "stay", "dead", "win", "tie"
		);
		
		// player state constants
		public const STATE_WAITING:int = 0;
		public const STATE_PLAYING:int = 1;
		public const STATE_STAYING:int = 2;
		public const STATE_DEAD:int = 3;
		public const STATE_WIN:int = 4;
		public const STATE_TIE:int = 5;
		
// *********************
// ***** VARIABLES *****
// *********************

		protected var _isHuman:Boolean;
		protected var _dice:Array;
		protected var _playerState:int;
		protected var _points:int;
		
// ***********************
// ***** CONSTRUCTOR *****
// ***********************
		public function ThirtySixPlayer()
		{
			_dice = new Array();
			reset(true);
			human_btn.addEventListener(flash.events.MouseEvent.MOUSE_UP, 
					handleComputerHumanToggle);
			computer_btn.addEventListener(flash.events.MouseEvent.MOUSE_UP, 
					handleComputerHumanToggle);
			_isHuman = true;
			human_btn.visible = true;
			computer_btn.visible = false;
		}
		
// **************************************
// ***** Functions - Event handlers *****
// **************************************

		public function handleRoll(evt:Event)
		{			
			roll_btn.removeEventListener(flash.events.MouseEvent.MOUSE_UP, handleRoll);
			stay_btn.removeEventListener(flash.events.MouseEvent.MOUSE_UP, handleStay);
			var score:int = addDie();
			if (score <= 36)
				setPlayerState(STATE_WAITING);
			var evt:Event = new Event(PLAYER_DONE_EVENT);
			this.dispatchEvent(evt);
		}
		
		public function handleStay(evt:Event)
		{
			roll_btn.removeEventListener(flash.events.MouseEvent.MOUSE_UP, handleRoll);
			stay_btn.removeEventListener(flash.events.MouseEvent.MOUSE_UP, handleStay);
			setPlayerState(STATE_STAYING);
			var evt:Event = new Event(PLAYER_DONE_EVENT);
			this.dispatchEvent(evt);
		}
	
		public function handleComputerHumanToggle(evt:Event)
		{
			if (_isHuman == true) {
				_isHuman = false;
				human_btn.visible = false;
				human_btn.removeEventListener(flash.events.MouseEvent.MOUSE_UP, 
						handleComputerHumanToggle);
				computer_btn.visible = true;
				computer_btn.addEventListener(flash.events.MouseEvent.MOUSE_UP, 
						handleComputerHumanToggle);
				if (_playerState == STATE_PLAYING)
					makeAIMove();
			} else {
				_isHuman = true;
				human_btn.visible = true;
				human_btn.addEventListener(flash.events.MouseEvent.MOUSE_UP, 
						handleComputerHumanToggle);
				computer_btn.removeEventListener(flash.events.MouseEvent.MOUSE_UP, 
						handleComputerHumanToggle);
				computer_btn.visible = false;
			}
		}
		
// **************************
// ***** Functions - AI *****
// **************************

		protected function makeAIMove():void
		{
			var theScore:int = getScore();
			if (theScore < AI_THRESHOLD) {
				theScore = addDie();
				if (theScore <= 36)
					setPlayerState(STATE_WAITING);
			} else {
				setPlayerState(STATE_STAYING);
			}
			var evt:Event = new Event(PLAYER_DONE_EVENT);
			this.dispatchEvent(evt);
		}

// ***************************************
// ***** Functions - General control *****
// ***************************************

		public function getScore():int
		{
			var theScore:int = 0;
			
			for (var cntr:int = 0; cntr < _dice.length; ++cntr)
				theScore += _dice[cntr].getValue();
			
			return theScore;
		}
		
		public function setPlayerState(newState:int):void
		{
			_playerState = newState;
			gotoAndPlay(LABELS[newState]);
			if ( (newState == STATE_PLAYING) && (_isHuman == false) )
				makeAIMove();
			if (newState == STATE_WIN)
				_points += 2;
			else if (newState == STATE_TIE)
				_points += 1;
			points_shadow_txt.text = ""+_points;
			points_main_txt.text = ""+_points;
		}
		
		public function getPlayerState():int
		{
			return _playerState;
		}
		
		/**
		 * Resets the player in order to start a new round or game.
		 *
		 * @param score determines if the point score should be reset as well
		*/
		public function reset(score:Boolean = false):void
		{
			var temp:SixSidedDie;
			
			if (score == true)
				_points = 0;
			
			while (_dice.length > 0) {
				temp = _dice.pop();
				removeChild(temp);
			}
			score_shadow_txt.text = "0";
			score_main_txt.text = "0";
			setPlayerState(STATE_WAITING);
		}
		
		protected function addDie():int
		{
			var newDie:SixSidedDie = new SixSidedDie();
			newDie.quickRoll();
			_dice.push(newDie);
			addChild(newDie);
			return updateDisplay();
		}
		
		/**
		 * Updates the positions of the dice being displayed and while doing so
		 * recalculates the score
		*/
		protected function updateDisplay():int
		{
			var newScore:int  = 0;
			var numDice = _dice.length;
			var cntr:int, numRows:int, dicePerRow:int, dieSize:int;

			for (cntr = 0; cntr < DIE_LIMITS.length; ++cntr) {
				if (DIE_LIMITS[cntr] >= numDice)
				{
					numRows = DIE_NUMROWS[cntr];
					dicePerRow = DIE_DICEPERROW[cntr];
					dieSize = DIE_SIZE[cntr];
					break;
				}
			}
			
			var tx:int = OFFSET_X;
			var ty:int = OFFSET_Y;
			var colCount:int = 0;
			for (cntr = 0; cntr < numDice; ++cntr) {
				_dice[cntr].x = tx;
				_dice[cntr].y = ty;
				_dice[cntr].width = dieSize;
				_dice[cntr].height = dieSize;
				++colCount;
				if (colCount >= dicePerRow) {
					colCount = 0;
					tx = OFFSET_X;
					ty += dieSize;
					ty += VGAP
				} else {
					tx += dieSize;
					tx += HGAP;
				}
				newScore += _dice[cntr].getValue();
			}
			if (newScore > 36)
				setPlayerState(STATE_DEAD);
			score_shadow_txt.text = ""+newScore;
			score_main_txt.text = ""+newScore;
			return newScore;
		}
	} // end interface
} // end package

